import { createRoot } from "react-dom/client";
import { Suspense, lazy, StrictMode } from "react";
import "./index.css";

// Import critical dependencies right away
import "./lib/i18n"; // We need i18n during initial render
import initPerformanceMonitoring from "./lib/performance-monitor";
import { preconnect, dnsPrefetch, preloadScript } from "./lib/script-loader";
import initMobileViewportFixes, { initSafeAreaInsets, isMobileDevice } from "./lib/mobile-viewport-fix";
import initViewportPriorityLoader from "./lib/viewport-priority-loader";
import { setupHashNavigation } from "./lib/simple-scroll";

// Initialize viewport priority loader immediately before anything else
initViewportPriorityLoader();

// Continue with other optimizations after viewport priority is established
initPerformanceMonitoring();
initMobileViewportFixes();
initSafeAreaInsets();

// Initialize our hash-based navigation system
setupHashNavigation();

// Lazy load the App component with priority
const App = lazy(() => {
  // Mark this as high-priority in the loading queue
  return import(/* webpackPrefetch: true */ "./App");
});

// Browser API detection for optimized loading strategies
const supportsIntersectionObserver = 'IntersectionObserver' in window;
const supportsIdleCallback = 'requestIdleCallback' in window;

/**
 * Optimized application boot sequence
 * This implements a carefully sequenced startup to maximize performance:
 * 1. Critical rendering path optimization
 * 2. Resource prioritization
 * 3. Progressive enhancement
 * 4. Deferred loading of non-critical resources
 */
const bootApplication = () => {
  // STAGE 1: Inject critical CSS and preconnect to domains
  injectCriticalCSS();
  setupResourceHints();
  
  // STAGE 2: Preload and prefetch critical assets
  preloadCriticalAssets();
  
  // STAGE 3: Mount application with optimized loading strategy
  mountApplication();
  
  // STAGE 4: Schedule deferred operations
  scheduleDeferredOperations();
};

// Add critical CSS immediately to avoid FOUC and reduce CLS
const injectCriticalCSS = () => {
  const style = document.createElement('style');
  style.innerHTML = `
    :root {
      --font-sans: 'Poppins', system-ui, -apple-system, BlinkMacSystemFont, sans-serif;
      --font-heading: 'Poppins', system-ui, -apple-system, BlinkMacSystemFont, sans-serif;
      --primary: #0066CC;
      --mobile-nav-height: 56px;
      --header-height: 80px;
      --content-width: 1280px;
      --scrollbar-width: calc(100vw - 100%);
    }
    
    /* System font fallbacks with similar metrics to prevent layout shifts */
    @font-face {
      font-family: 'Poppins Fallback';
      src: local('Arial');
      size-adjust: 105%;
      ascent-override: 110%;
      descent-override: 25%;
      line-gap-override: 10%;
    }
    
    /* System font fallbacks with size-adjust to precisely match the metrics of loaded fonts */
    @font-face {
      font-family: 'Poppins Fallback';
      font-style: normal;
      font-weight: 400;
      src: local('Arial');
      size-adjust: 105%;
      ascent-override: 110%;
      descent-override: 25%;
      line-gap-override: 10%;
    }
    
    /* Font definitions with size metrics to prevent CLS */
    @font-face {
      font-family: 'Poppins';
      font-style: normal;
      font-weight: 400;
      font-display: optional; /* Use fallback font until custom font loads */
      src: local('Poppins Regular'), local('Poppins-Regular'),
           url('https://fonts.gstatic.com/s/poppins/v15/pxiEyp8kv8JHgFVrJJfecg.woff2') format('woff2');
      font-stretch: normal;
      unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
    }
    
    @font-face {
      font-family: 'Poppins';
      font-style: normal;
      font-weight: 500;
      font-display: optional;
      src: local('Poppins Medium'), local('Poppins-Medium'),
           url('https://fonts.gstatic.com/s/poppins/v15/pxiByp8kv8JHgFVrLGT9Z1xlFQ.woff2') format('woff2');
      font-stretch: normal;
      unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
    }
    
    @font-face {
      font-family: 'Poppins';
      font-style: normal;
      font-weight: 600;
      font-display: optional;
      src: local('Poppins SemiBold'), local('Poppins-SemiBold'),
           url('https://fonts.gstatic.com/s/poppins/v15/pxiByp8kv8JHgFVrLEj6Z1xlFQ.woff2') format('woff2');
      font-stretch: normal;
      unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
    }

    @font-face {
      font-family: 'Poppins';
      font-style: normal;
      font-weight: 700;
      font-display: optional;
      src: local('Poppins Bold'), local('Poppins-Bold'),
           url('https://fonts.gstatic.com/s/poppins/v15/pxiByp8kv8JHgFVrLCz7Z1xlFQ.woff2') format('woff2');
      font-stretch: normal;
      unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
    }
    
    /* Size-adjust fallbacks for brand typography to prevent shifts */
    h1, h2, h3, h4, h5, h6 {
      font-family: var(--font-heading);
      margin: 0;
      text-wrap: balance; /* Balance text lines to prevent shifts */
      line-height: 1.2; /* Set explicit line height to prevent shifts */
    }
    
    body {
      font-family: var(--font-sans);
      margin: 0;
      padding: 0;
      background-color: #f9fafb;
      color: #111827;
      text-rendering: optimizeSpeed;
      -webkit-font-smoothing: antialiased;
      overflow-y: scroll; /* Always show scrollbar to prevent width shifts */
      overflow-x: hidden; /* Prevent horizontal scroll */
      width: calc(100% - var(--scrollbar-width)); /* Compensate for scrollbar */
    }

    /* Ensure core layout doesn't shift during load */
    #root {
      display: flex;
      flex-direction: column;
      min-height: 100vh;
    }
    
    /* Pre-set size for hero images and container to reduce CLS */
    .hero-section {
      position: relative;
      min-height: 70vh;
      overflow: hidden;
      contain: layout; /* Contain layout changes to this element */
    }
    
    /* Reserve space for images that will load later to prevent layout shifts */
    .image-container {
      position: relative;
      overflow: hidden;
      background-color: rgba(229, 231, 235, 0.5);
      contain: size layout; /* Contain size and layout changes */
      aspect-ratio: 16/9; /* Default aspect ratio */
    }
    
    /* Reserve space for specific content */
    .hero-image {
      aspect-ratio: 16/9;
      background-color: rgba(229, 231, 235, 0.5);
      height: auto;
      width: 100%;
      display: block; /* Prevent inline display issues */
      contain: size layout; /* Contain size and layout changes */
    }
    
    /* Fixed header dimensions to prevent jumps */
    .header-container {
      height: var(--header-height);
      background-color: rgba(255, 255, 255, 0.9);
      backdrop-filter: blur(8px);
      -webkit-backdrop-filter: blur(8px);
      position: sticky;
      top: 0;
      z-index: 30;
      contain: size layout; /* Contain size and layout changes */
    }
    
    /* Set content-visibility to improve rendering performance */
    .offscreen-content {
      content-visibility: auto;
      contain-intrinsic-size: 400px;
      contain: size layout; /* Contain size and layout changes */
    }
    
    /* Add containing block for position: fixed elements */
    .fixed-container {
      transform: translateZ(0);
      will-change: transform;
      backface-visibility: hidden;
    }

    /* Advanced loading animation with transform-only animations */
    .app-loading {
      display: flex;
      justify-content: center;
      align-items: center;
      height: 100vh;
      width: 100%;
      background-color: #f9fafb;
      position: fixed;
      top: 0;
      left: 0;
      z-index: 9999;
      contain: size layout; /* Contain size and layout changes */
    }

    .app-loading-spinner {
      width: 50px;
      height: 50px;
      border: 3px solid rgba(0, 102, 204, 0.2);
      border-radius: 50%;
      border-top-color: var(--primary);
      animation: spin 1s cubic-bezier(0.4, 0, 0.2, 1) infinite;
      will-change: transform;
    }

    .app-loading-logo {
      position: absolute;
      top: 50%;
      left: 50%;
      transform: translate(-50%, -50%);
      width: 120px;
      opacity: 0.9;
    }

    @keyframes spin {
      to { transform: rotate(360deg); }
    }
    
    /* Fix for Safari CLS issues with images */
    img {
      height: auto;
      max-width: 100%;
      aspect-ratio: attr(width) / attr(height);
      display: block; /* Block display prevents line-height issues */
      contain: size layout style; /* Contain layout changes */
    }
    
    /* Prevent layout shifts for form elements */
    input, select, textarea, button {
      font-family: inherit;
      box-sizing: border-box;
    }
    
    /* Optimize animations to use transforms */
    .animate-transform {
      transition-property: transform, opacity !important;
      will-change: transform, opacity;
    }

    /* Fix intrinsic width/height for common UI elements */
    .ui-card {
      contain: content;
      contain-intrinsic-size: 0 300px;
    }
    
    /* Size containers for nav elements */
    .nav-container {
      height: var(--mobile-nav-height);
      contain: size layout; 
    }
    
    /* Set explicit sizes for navigation elements */
    .nav-link {
      height: 44px; /* Touch-friendly size */
      display: flex;
      align-items: center;
    }
    
    /* Pre-allocate space for modal content */
    .modal-content {
      position: fixed;
      contain: size layout;
    }
  `;
  document.head.appendChild(style);
};

// Setup resource hints for better performance
const setupResourceHints = () => {
  // DNS prefetch for third-party domains
  dnsPrefetch('https://fonts.googleapis.com');
  dnsPrefetch('https://fonts.gstatic.com');
  dnsPrefetch('https://js.stripe.com');
  
  // Preconnect to critical domains
  preconnect('https://fonts.googleapis.com');
  preconnect('https://fonts.gstatic.com', true);
  preconnect('https://js.stripe.com', true);
  
  // Preload critical fonts to reduce CLS
  const fontPreloads = [
    { 
      href: 'https://fonts.gstatic.com/s/poppins/v15/pxiEyp8kv8JHgFVrJJfecg.woff2',
      type: 'font/woff2',
      weight: '400',
      display: 'optional' 
    },
    { 
      href: 'https://fonts.gstatic.com/s/poppins/v15/pxiByp8kv8JHgFVrLGT9Z1xlFQ.woff2',
      type: 'font/woff2',
      weight: '500',
      display: 'optional'
    },
    { 
      href: 'https://fonts.gstatic.com/s/poppins/v15/pxiByp8kv8JHgFVrLCz7Z1xlFQ.woff2',
      type: 'font/woff2',
      weight: '700',
      display: 'optional'
    }
  ];
  
  // Add font preload links
  fontPreloads.forEach(font => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = font.href;
    link.as = 'font';
    link.type = font.type;
    link.crossOrigin = 'anonymous';
    
    // Add font-display as a data attribute since it's not standard in preload
    link.setAttribute('data-font-display', font.display);
    link.setAttribute('data-font-weight', font.weight);
    
    document.head.appendChild(link);
    
    // Also add the actual font-face to ensure it's loaded with the right settings
    const style = document.createElement('style');
    style.textContent = `
      @font-face {
        font-family: 'Poppins';
        font-style: normal;
        font-weight: ${font.weight};
        font-display: ${font.display};
        src: url(${font.href}) format('woff2');
        unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
      }
    `;
    document.head.appendChild(style);
  });
};

// Define type for preload resources
type PreloadResource = {
  href: string;
  as: 'image' | 'style' | 'script' | 'font' | 'fetch';
  type?: string;
  crossOrigin?: string;
};

// Preload critical resources that are needed for LCP
const preloadCriticalAssets = () => {
  // Determine critical assets based on entry point
  const entryPath = window.location.pathname;
  const isMobile = window.innerWidth < 768;
  
  // Base assets needed everywhere
  const criticalAssets: PreloadResource[] = [
    { href: '/assets/logo.svg', as: 'image' },
  ];
  
  // Add optimized images for faster loading
  if (entryPath === '/' || entryPath === '') {
    // Use WebP for mobile and optimized images for desktop
    if (isMobile) {
      criticalAssets.push(
        { href: '/assets/optimized/Ellie_Heroshot_grey_background_mobile.png', as: 'image' },
        { href: '/assets/optimized/ellie_device_setup_mobile.webp', as: 'image' }
      );
    } else {
      criticalAssets.push(
        { href: '/assets/optimized/Ellie_Heroshot_grey_background_tablet.webp', as: 'image' },
        { href: '/assets/optimized/ellie_device_setup_tablet.webp', as: 'image' }
      );
    }
  }
  
  // Create preload links for all critical assets
  criticalAssets.forEach(resource => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = resource.href;
    link.as = resource.as;
    if (resource.as === 'font') {
      link.type = 'font/woff2';
      link.crossOrigin = 'anonymous';
    }
    document.head.appendChild(link);
  });
  
  // Prefetch main JavaScript chunks
  preloadScript('/src/App.tsx');
  preloadScript('/src/pages/home.tsx');
};

// Mount application with optimized suspense
const mountApplication = () => {
  const rootElement = document.getElementById("root");
  if (!rootElement) return;
  
  const root = createRoot(rootElement);
  
  // Use empty fallback to avoid showing any loader - critical sections are already prioritized
  const EmptyFallback = () => null;
  
  // Render with Suspense - empty fallback for immediate content display
  root.render(
    <StrictMode>
      <Suspense fallback={<EmptyFallback />}>
        <App />
      </Suspense>
    </StrictMode>
  );
  
  // Prioritize initial rendering
  document.documentElement.classList.add('prioritized-rendering');
};

// Schedule deferred operations for non-critical tasks
const scheduleDeferredOperations = () => {
  const deferredLoad = async () => {
    try {
      // Register service worker for PWA capabilities
      if ('serviceWorker' in navigator) {
        // Only register service worker after page is fully loaded
        window.addEventListener('load', () => {
          navigator.serviceWorker.register('/sw.js', { scope: '/' }).catch(() => {});
        });
      }
      
      // Perform feature detection - no need to load polyfills
      // Most modern browsers support IntersectionObserver
      if (!supportsIntersectionObserver) {
        console.warn('IntersectionObserver API not supported. Some features may not work correctly.');
      }
      
      // Load non-critical modules
      const { initMarketingAgent } = await import("./lib/marketing-agent");
      initMarketingAgent();
      
      // Prefetch likely navigation paths in idle time
      prefetchLikelyNavigation();
    } catch (e) {
      // Silent fail - these are enhancements only
      console.debug('Error in deferred operations:', e);
    }
  };
  
  // Execute deferred operations during idle time
  if (supportsIdleCallback) {
    (window as any).requestIdleCallback(deferredLoad, { timeout: 5000 });
  } else {
    // Fallback for browsers without requestIdleCallback
    setTimeout(deferredLoad, 3000);
  }
};

// Prefetch content for likely user navigation paths
const prefetchLikelyNavigation = () => {
  const entryPath = window.location.pathname;
  
  // For homepage, preload popular navigation destinations
  if (entryPath === '/' || entryPath === '') {
    // These will be loaded in idle time based on likely user paths
    const likelyPaths = [
      () => import('./pages/webshop'),
      () => import('./pages/product-detail'),
    ];
    
    // Only preload these chunks in idle time
    if (supportsIdleCallback) {
      (window as any).requestIdleCallback(() => {
        likelyPaths.forEach(loader => loader());
      }, { timeout: 10000 });
    }
  }
};

// Start the optimized boot sequence
bootApplication();
